@@ -53,6 +53,7 @@ struct image_dispatcher
     image_dispatcher(int start_x, int start_y,
                           int width, int height,
                           double scale_x, double scale_y,
+                          double offset_x, double offset_y,
                           scaling_method_e method, double filter_factor,
                           double opacity, composite_mode_e comp_op,
                           raster_symbolizer const& sym, feature_impl const& feature,
@@ -63,15 +64,17 @@ struct image_dispatcher
           height_(height),
           scale_x_(scale_x),
           scale_y_(scale_y),
+          offset_x_(offset_x),
+          offset_y_(offset_y),
           method_(method),
           filter_factor_(filter_factor),
-        opacity_(opacity),
-        comp_op_(comp_op),
-        sym_(sym),
-        feature_(feature),
-        composite_(composite),
-        nodata_(nodata),
-        need_scaling_(need_scaling) {}
+          opacity_(opacity),
+          comp_op_(comp_op),
+          sym_(sym),
+          feature_(feature),
+          composite_(composite),
+          nodata_(nodata),
+          need_scaling_(need_scaling) {}
 
     void operator() (image_null const&) const {}  //no-op
     void operator() (image_rgba8 const& data_in) const
@@ -79,7 +82,7 @@ struct image_dispatcher
         if (need_scaling_)
         {
             image_rgba8 data_out(width_, height_, true, true);
-            scale_image_agg(data_out, data_in,  method_, scale_x_, scale_y_, 0.0, 0.0, filter_factor_, nodata_);
+            scale_image_agg(data_out, data_in,  method_, scale_x_, scale_y_, offset_x_, offset_y_, filter_factor_, nodata_);
             composite_(data_out, comp_op_, opacity_, start_x_, start_y_);
         }
         else
@@ -97,7 +100,7 @@ struct image_dispatcher
         if (need_scaling_)
         {
             image_type data_out(width_, height_);
-            scale_image_agg(data_out, data_in,  method_, scale_x_, scale_y_, 0.0, 0.0, filter_factor_, nodata_);
+            scale_image_agg(data_out, data_in,  method_, scale_x_, scale_y_, offset_x_, offset_y_, filter_factor_, nodata_);
             if (colorizer) colorizer->colorize(dst, data_out, nodata_, feature_);
         }
         else
@@ -114,6 +117,8 @@ private:
     int height_;
     double scale_x_;
     double scale_y_;
+    double offset_x_;
+    double offset_y_;
     scaling_method_e method_;
     double filter_factor_;
     double opacity_;
@@ -210,12 +215,18 @@ void render_raster_symbolizer(raster_symbolizer const& sym,
     if (source)
     {
         box2d<double> target_ext = box2d<double>(source->ext_);
-        prj_trans.backward(target_ext, PROJ_ENVELOPE_POINTS);
+        box2d<double> target_query_ext = box2d<double>(source->query_ext_);
+        if (!prj_trans.equal())
+        {
+            prj_trans.backward(target_ext, PROJ_ENVELOPE_POINTS);
+            prj_trans.backward(target_query_ext, PROJ_ENVELOPE_POINTS);
+        }
         box2d<double> ext = common.t_.forward(target_ext);
-        int start_x = static_cast<int>(std::floor(ext.minx()+.5));
-        int start_y = static_cast<int>(std::floor(ext.miny()+.5));
-        int end_x = static_cast<int>(std::floor(ext.maxx()+.5));
-        int end_y = static_cast<int>(std::floor(ext.maxy()+.5));
+        box2d<double> query_ext = common.t_.forward(target_query_ext);
+        int start_x = static_cast<int>(std::floor(query_ext.minx()+.5));
+        int start_y = static_cast<int>(std::floor(query_ext.miny()+.5));
+        int end_x = static_cast<int>(std::floor(query_ext.maxx()+.5));
+        int end_y = static_cast<int>(std::floor(query_ext.maxy()+.5));
         int raster_width = end_x - start_x;
         int raster_height = end_y - start_y;
         if (raster_width > 0 && raster_height > 0)
@@ -236,17 +247,20 @@ void render_raster_symbolizer(raster_symbolizer const& sym,
 
             if (!prj_trans.equal())
             {
-                double offset_x = ext.minx() - start_x;
-                double offset_y = ext.miny() - start_y;
+                // This path does not currently work and is still being figured out. 
+                double offset_x = query_ext.minx() - start_x;
+                double offset_y = query_ext.miny() - start_y;
                 unsigned mesh_size = static_cast<unsigned>(get<value_integer>(sym,keys::mesh_size,feature, common.vars_, 16));
                 detail::image_warp_dispatcher<F> dispatcher(prj_trans, start_x, start_y, raster_width, raster_height,
-                                                                 target_ext, source->ext_, offset_x, offset_y, mesh_size,
+                                                                 target_query_ext, source->ext_, offset_x, offset_y, mesh_size,
                                                                  scaling_method, source->get_filter_factor(),
                                                                  opacity, comp_op, sym, feature, composite, source->nodata());
                 util::apply_visitor(dispatcher, source->data_);
             }
             else
             {
+                double offset_x = query_ext.minx() - ext.minx();
+                double offset_y = query_ext.miny() - ext.miny();
                 double image_ratio_x = ext.width() / source->data_.width();
                 double image_ratio_y = ext.height() / source->data_.height();
                 double eps = 1e-5;
@@ -256,6 +270,7 @@ void render_raster_symbolizer(raster_symbolizer const& sym,
                      (std::abs(start_y) > eps);
                 detail::image_dispatcher<F> dispatcher(start_x, start_y, raster_width, raster_height,
                                                             image_ratio_x, image_ratio_y,
+                                                            offset_x, offset_y,
                                                             scaling_method, source->get_filter_factor(),
                                                             opacity, comp_op, sym, feature, composite, source->nodata(), scale);
                 util::apply_visitor(dispatcher, source->data_);
